#include "commands.h"

/* This file defines the commands structure.
 * This is the canonical reference for all commands
 * used. This file can be used - unchanged - on both the quad side
 * and on the ground station side. The commands.h file is also entirely
 * portable and may be used unchanged.
 *
 * This file (commands.c) and the matching header (commands.h)
 * are fully portable (quad + groundStation).
 * 
 * To use this file, three non-portable files must also exist:
 *  - callbacks.h. Typedef for command_cb
 *  - cb_default.h. Implementation of cb_default.
 *  - callbacks.c file. Contains implemented callbacks.
 *
 * There are two mandatory things that must be implemented in
 * other files for this to work: First, in callbacks.h, create a typedef
 * for command_cb. See the callbacks.h of the ground station for an
 * example.
 *
 * Second, in cb_default.h, implement the function
 * cb_default. This function should do nothing; it will be the 
 * default action for an unimplemented callback. Note that because
 * the function is implemented in the .h file, cb_default.h MUST NOT
 * be included in any other file!
 *
 * To implement callbacks, simply define them in callbacks.c.
 *
 */

/* List of callbacks. DO NOT MODIFY THESE IN THIS FILE - 
 * Simply implement a function with the same name
 * in a different file (callbacks.c) and these will
 * be overridden.
 */

/* Grab the default callback configuration */
#include "cb_default.h"

command_cb cb_debug __attribute__((weak, alias("cb_default")));
command_cb cb_setyawp __attribute__((weak, alias("cb_default")));

/* Command structure */
struct MessageType MessageTypes[MAX_TYPE] =
{
	// DEBUG
	{
		// Message Type ID
		0x00,

		// Debug Subtypes
		{
			// NONE subtype
			{
				// ID
				0x00,
				// Command text
				"debug",
				// Type of the command data
				stringType,
				// Function pointer
				&cb_debug
			}
		}
	},

	// CALIBRATION
	{
		// Message Type ID
		0x01,

		// Calibration Subtypes (PID coefficients)
		{
			// yaw p constant subtype
			{
				// ID
				0x00,
				// Command text
				"setyawp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// yaw i constant subtype
			{
				// ID
				0x01,
				// Command text
				"setyawi",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// yaw d constant subtype
			{
				// ID
				0x02,
				// Command text
				"setyawd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// roll p constant subtype
			{
				// ID
				0x03,
				// Command text
				"setrollp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// roll i constant subtype
			{
				// ID
				0x04,
				// Command text
				"setrolli",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// roll d constant subtype
			{
				// ID
				0x05,
				// Command text
				"setrolld",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitch p constant subtype
			{
				// ID
				0x06,
				// Command text
				"setpitchp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitch i constant subtype
			{
				// ID
				0x07,
				// Command text
				"setpitchi",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitch d constant subtype
			{
				// ID
				0x08,
				// Command text
				"setpitchd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// yawrate p constant subtype
			{
				// ID
				0x09,
				// Command text
				"setyawratep",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// yawrate i constant subtype
			{
				// ID
				0x0A,
				// Command text
				"setyawratei",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// yawrate d constant subtype
			{
				// ID
				0x0B,
				// Command text
				"setyawrated",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// rollrate p constant subtype
			{
				// ID
				0x0C,
				// Command text
				"setrollratep",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// rollrate i constant subtype
			{
				// ID
				0x0D,
				// Command text
				"setrollratei",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// rollrate d constant subtype
			{
				// ID
				0x0E,
				// Command text
				"setrollrated",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitchrate p constant subtype
			{
				// ID
				0x0F,
				// Command text
				"setpitchratep",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitchrate i constant subtype
			{
				// ID
				0x10,
				// Command text
				"setpitchratei",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitchrate d constant subtype
			{
				// ID
				0x11,
				// Command text
				"setpitchrated",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// height p constant subtype
			{
				// ID
				0x12,
				// Command text
				"setheightp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// height i constant subtype
			{
				// ID
				0x13,
				// Command text
				"setheighti",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// height d constant subtype
			{
				// ID
				0x14,
				// Command text
				"setheightd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// lat p constant subtype
			{
				// ID
				0x15,
				// Command text
				"setlatp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// lat i constant subtype
			{
				// ID
				0x16,
				// Command text
				"setlati",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// lat d constant subtype
			{
				// ID
				0x17,
				// Command text
				"setlatd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// long p constant subtype
			{
				// ID
				0x18,
				// Command text
				"setlongp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// long i constant subtype
			{
				// ID
				0x19,
				// Command text
				"setlongi",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// long d constant subtype
			{
				// ID
				0x1A,
				// Command text
				"setlongd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			}
		}
	},

	// REQUEST
	{
		// Message Type ID
		0x02,

		// Request Subtypes
		{
			// accelerometer subtype
			{
				// ID
				0x00,
				// Command text
				"getaccel",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// gyroscope subtype
			{
				// ID
				0x01,
				// Command text
				"getgyro",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitch angle subtype
			{
				// ID
				0x02,
				// Command text
				"getpitchangle",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// roll angle subtype
			{
				// ID
				0x03,
				// Command text
				"getrollangle",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get yaw setpoint subtype
			{
				// ID
				0x04,
				// Command text
				"getyaw",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get yaw p constant subtype
			{
				// ID
				0x05,
				// Command text
				"getyawp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get yaw d constant subtype
			{
				// ID
				0x06,
				// Command text
				"getyawd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get roll setpoint subtype
			{
				// ID
				0x07,
				// Command text
				"getroll",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get roll p constant subtype
			{
				// ID
				0x08,
				// Command text
				"getrollp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// getroll d constant subtype
			{
				// ID
				0x09,
				// Command text
				"getrolld",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get pitch setpoint subtype
			{
				// ID
				0x0A,
				// Command text
				"getpitch",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get pitch p constant subtype
			{
				// ID
				0x0B,
				// Command text
				"getpitchp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get pitch d constant subtype
			{
				// ID
				0x0C,
				// Command text
				"getpitchd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get throttle setpoint subtype
			{
				// ID
				0x0D,
				// Command text
				"getthrottle",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get throttle p constant subtype
			{
				// ID
				0x0E,
				// Command text
				"getthrottlep",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get throttle i constant subtype
			{
				// ID
				0x0F,
				// Command text
				"getthrottlei",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// get throttle d constant subtype
			{
				// ID
				0x10,
				// Command text
				"getthrottled",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			}
		}
	},

	// RESPONSE
	{
		// Message Type ID
		0x03,

		// Response Subtypes
		{
			// accelerometer subtype
			{
				// ID
				0x00,
				// Command text
				"respaccel",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// gyroscope subtype
			{
				// ID
				0x01,
				// Command text
				"respgyro",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// pitch angle subtype
			{
				// ID
				0x02,
				// Command text
				"resppitchangle",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// roll angle subtype
			{
				// ID
				0x03,
				// Command text
				"resprollangle",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp yaw setpoint subtype
			{
				// ID
				0x04,
				// Command text
				"respyaw",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp yaw p constant subtype
			{
				// ID
				0x05,
				// Command text
				"respyawp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp yaw d constant subtype
			{
				// ID
				0x06,
				// Command text
				"respyawd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp roll setpoint subtype
			{
				// ID
				0x07,
				// Command text
				"resproll",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp roll p constant subtype
			{
				// ID
				0x08,
				// Command text
				"resprollp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resproll d constant subtype
			{
				// ID
				0x09,
				// Command text
				"resprolld",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp pitch setpoint subtype
			{
				// ID
				0x0A,
				// Command text
				"resppitch",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp pitch p constant subtype
			{
				// ID
				0x0B,
				// Command text
				"resppitchp",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp pitch d constant subtype
			{
				// ID
				0x0C,
				// Command text
				"resppitchd",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp throttle setpoint subtype
			{
				// ID
				0x0D,
				// Command text
				"respthrottle",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp throttle p constant subtype
			{
				// ID
				0x0E,
				// Command text
				"respthrottlep",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp throttle i constant subtype
			{
				// ID
				0x0F,
				// Command text
				"respthrottlei",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			},
			// resp throttle d constant subtype
			{
				// ID
				0x10,
				// Command text
				"respthrottled",
				// Type of the command data
				floatType,
				// Function pointer
				NULL
			}
		}
	},

	// UPDATE
	{
		// Message Type ID
		0x04,

		// Update Subtypes
		{
			// NONE subtype
			{
				// ID
				0x00,
				// Command text
				"update",
				// Type of the command data
				stringType,
				// Function pointer
				NULL
			}
		}
	},

	// LOG
	{
		// Message Type ID
		0x05,

		// Log Subtypes
		{
			// NONE subtype
			{
				// ID
				0x00,
				// Command text
				"log",
				// Type of the command data
				stringType,
				// Function pointer
				NULL
			},
			// Response subtype
			{
				// ID
				0x01,
				// Command text
				"response",
				// Type of the command data
				stringType,
				// Function pointer
				NULL
			}
		}
	},

};
